"""File system watcher."""

import time
from typing import Callable, Set, Optional
from watchdog.observers import Observer
from pathlib import Path
from sourcescribe.watch.handler import ChangeHandler
from sourcescribe.config.models import WatchConfig, RepositoryConfig
from sourcescribe.utils.logger import get_logger


class FileWatcher:
    """Watches file system for changes."""
    
    def __init__(
        self,
        root_path: str,
        callback: Callable[[Set[str]], None],
        watch_config: Optional[WatchConfig] = None,
        repo_config: Optional[RepositoryConfig] = None,
    ):
        """
        Initialize file watcher.
        
        Args:
            root_path: Root directory to watch
            callback: Function to call with changed files
            watch_config: Watch configuration
            repo_config: Repository configuration
        """
        self.root_path = Path(root_path).resolve()
        self.callback = callback
        self.watch_config = watch_config or WatchConfig()
        self.repo_config = repo_config or RepositoryConfig()
        
        self.logger = get_logger(__name__)
        self.observer: Optional[Observer] = None
        self.handler: Optional[ChangeHandler] = None
        self._running = False
    
    def start(self) -> None:
        """Start watching for file changes."""
        if self._running:
            self.logger.warning("Watcher is already running")
            return
        
        if not self.watch_config.enabled:
            self.logger.info("Watch mode is disabled")
            return
        
        self.logger.info(f"Starting file watcher on: {self.root_path}")
        
        # Create handler
        self.handler = ChangeHandler(
            callback=self.callback,
            debounce_seconds=self.watch_config.debounce_seconds,
            include_patterns=set(self.repo_config.include_patterns),
            exclude_patterns=set(self.repo_config.exclude_patterns),
        )
        
        # Create and start observer
        self.observer = Observer()
        self.observer.schedule(
            self.handler,
            str(self.root_path),
            recursive=self.watch_config.recursive,
        )
        self.observer.start()
        self._running = True
        
        self.logger.info("File watcher started")
    
    def stop(self) -> None:
        """Stop watching for file changes."""
        if not self._running:
            return
        
        self.logger.info("Stopping file watcher")
        
        if self.observer:
            self.observer.stop()
            self.observer.join(timeout=5.0)
        
        self._running = False
        self.logger.info("File watcher stopped")
    
    def run(self) -> None:
        """
        Run watcher in blocking mode.
        
        Watches for changes and processes them. Press Ctrl+C to stop.
        """
        self.start()
        
        if not self._running:
            return
        
        try:
            self.logger.info("Watching for changes... Press Ctrl+C to stop")
            
            while self._running:
                # Check for pending changes
                if self.handler and self.handler.has_pending_changes():
                    self.handler.process_pending_changes()
                
                # Sleep briefly
                time.sleep(0.5)
        
        except KeyboardInterrupt:
            self.logger.info("Received interrupt signal")
        finally:
            self.stop()
    
    def is_running(self) -> bool:
        """Check if watcher is running."""
        return self._running
    
    def __enter__(self):
        """Context manager entry."""
        self.start()
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit."""
        self.stop()



"""Ollama local LLM provider."""

import requests
from typing import List, Optional, Any, Iterator, Dict
from sourcescribe.api.base import BaseLLMProvider, LLMMessage, LLMResponse


class OllamaProvider(BaseLLMProvider):
    """Ollama local LLM provider."""
    
    def __init__(self, *args, **kwargs):
        """Initialize Ollama provider."""
        super().__init__(*args, **kwargs)
        
        # Set default base URL if not provided
        if not self.base_url:
            self.base_url = "http://localhost:11434"
        
        # Set default model if not provided
        if not self.model:
            self.model = "llama2"
        
        self.api_url = f"{self.base_url}/api"
        
        # Check if Ollama is running
        self._check_connection()
    
    def _check_connection(self) -> None:
        """Check if Ollama server is accessible."""
        try:
            response = requests.get(f"{self.base_url}/api/tags", timeout=5)
            response.raise_for_status()
        except requests.exceptions.ConnectionError:
            raise ConnectionError(
                f"Cannot connect to Ollama at {self.base_url}\n"
                "Make sure Ollama is installed and running:\n"
                "  1. Install: https://ollama.ai/download\n"
                "  2. Start: ollama serve\n"
                "  3. Pull a model: ollama pull llama2"
            )
        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                raise ConnectionError(
                    f"Ollama server at {self.base_url} is not responding correctly.\n"
                    "Make sure you're running a compatible version of Ollama."
                )
            raise
    
    def generate(
        self,
        messages: List[LLMMessage],
        system_prompt: Optional[str] = None,
        **kwargs: Any
    ) -> LLMResponse:
        """
        Generate response using Ollama.
        
        Args:
            messages: Conversation messages
            system_prompt: System prompt
            **kwargs: Additional parameters
            
        Returns:
            LLMResponse
        """
        self.validate_config()
        
        # Build prompt from messages
        prompt = self._build_prompt(messages, system_prompt)
        
        # Prepare request
        data = {
            "model": self.model,
            "prompt": prompt,
            "stream": False,
            "options": {
                "temperature": kwargs.get("temperature", self.temperature),
                "num_predict": kwargs.get("max_tokens", self.max_tokens),
            }
        }
        
        # Make API call
        try:
            response = requests.post(
                f"{self.api_url}/generate",
                json=data,
                timeout=self.timeout,
            )
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                # Try to get list of available models
                try:
                    tags_response = requests.get(f"{self.api_url}/tags", timeout=5)
                    if tags_response.ok:
                        models_data = tags_response.json()
                        available_models = [m.get('name', '') for m in models_data.get('models', [])]
                        if available_models:
                            raise ValueError(
                                f"Model '{self.model}' not found. Available models: {', '.join(available_models)}\n"
                                f"Pull the model with: ollama pull {self.model}"
                            )
                except:
                    pass
                raise ConnectionError(
                    f"Ollama API endpoint not found.\n"
                    f"Make sure:\n"
                    f"  1. Ollama is running: ollama serve\n"
                    f"  2. Model '{self.model}' is pulled: ollama pull {self.model}\n"
                    f"  3. You're using a compatible Ollama version"
                )
            raise
        
        result = response.json()
        
        return LLMResponse(
            content=result.get("response", ""),
            model=self.model,
            usage={
                "prompt_tokens": result.get("prompt_eval_count", 0),
                "completion_tokens": result.get("eval_count", 0),
                "total_tokens": result.get("prompt_eval_count", 0) + result.get("eval_count", 0),
            },
            finish_reason=result.get("done_reason"),
            raw_response=result,
        )
    
    def generate_streaming(
        self,
        messages: List[LLMMessage],
        system_prompt: Optional[str] = None,
        **kwargs: Any
    ) -> Iterator[str]:
        """
        Generate streaming response using Ollama.
        
        Args:
            messages: Conversation messages
            system_prompt: System prompt
            **kwargs: Additional parameters
            
        Yields:
            Response chunks
        """
        self.validate_config()
        
        # Build prompt from messages
        prompt = self._build_prompt(messages, system_prompt)
        
        # Prepare request
        data = {
            "model": self.model,
            "prompt": prompt,
            "stream": True,
            "options": {
                "temperature": kwargs.get("temperature", self.temperature),
                "num_predict": kwargs.get("max_tokens", self.max_tokens),
            }
        }
        
        # Stream response
        response = requests.post(
            f"{self.api_url}/generate",
            json=data,
            timeout=self.timeout,
            stream=True,
        )
        response.raise_for_status()
        
        for line in response.iter_lines():
            if line:
                import json
                result = json.loads(line)
                if "response" in result:
                    yield result["response"]
    
    def _build_prompt(self, messages: List[LLMMessage], system_prompt: Optional[str] = None) -> str:
        """Build a single prompt string from messages."""
        parts = []
        
        if system_prompt:
            parts.append(f"System: {system_prompt}\n")
        
        for msg in messages:
            role = msg.role.capitalize()
            parts.append(f"{role}: {msg.content}\n")
        
        parts.append("Assistant:")
        
        return "\n".join(parts)
    
    def list_models(self) -> List[Dict[str, Any]]:
        """
        List available Ollama models.
        
        Returns:
            List of model information dictionaries
        """
        response = requests.get(f"{self.api_url}/tags", timeout=10)
        response.raise_for_status()
        return response.json().get("models", [])



"""Mermaid diagram generation."""

from typing import List, Dict, Any, Optional
from dataclasses import dataclass


@dataclass
class DiagramNode:
    """Represents a node in a diagram."""
    id: str
    label: str
    type: str  # class, function, module, etc.
    
@dataclass
class DiagramEdge:
    """Represents an edge in a diagram."""
    source: str
    target: str
    label: Optional[str] = None


class DiagramGenerator:
    """Generate Mermaid.js diagrams from code structure."""
    
    def generate_architecture_diagram(
        self,
        modules: List[Dict[str, Any]],
        title: str = "System Architecture"
    ) -> str:
        """
        Generate architecture diagram with smart filtering and grouping.
        
        Args:
            modules: List of module information
            title: Diagram title
            
        Returns:
            Mermaid diagram as string
        """
        # Group modules by directory/layer
        grouped_modules = self._group_modules_by_layer(modules)
        
        # Select most important modules to show (limit to avoid overwhelming diagram)
        important_modules = self._select_important_modules(modules, max_nodes=50)
        
        # Build module name to index map
        module_index = {m['name']: i for i, m in enumerate(important_modules)}
        
        lines = [
            "```mermaid",
            "graph TD",
            ""
        ]
        
        # Add grouped nodes with styling
        for layer, layer_modules in grouped_modules.items():
            # Only show modules that are in our important list
            layer_important = [m for m in layer_modules if m['name'] in module_index]
            if not layer_important:
                continue
                
            lines.append(f"    %% {layer} Layer")
            for module in layer_important:
                idx = module_index[module['name']]
                node_id = f"M{idx}"
                module_name = module.get('name', f'Module{idx}')
                
                # Add styling based on module characteristics
                style = self._get_node_style(module)
                lines.append(f"    {node_id}[{module_name}]{style}")
            lines.append("")
        
        # Add relationships (filtered to show only significant ones)
        edges_added = 0
        edges_by_importance = []
        unmatched_deps = set()  # Track dependencies that don't match any module
        
        for i, module in enumerate(important_modules):
            deps = module.get('dependencies', [])
            for dep in deps:
                if dep in module_index:
                    j = module_index[dep]
                    # Calculate importance (modules with more deps = more important edges)
                    importance = len(module.get('dependencies', []))
                    edges_by_importance.append((importance, i, j, module['name'], dep))
                else:
                    # Track unmatched for debugging
                    unmatched_deps.add(dep)
        
        # Sort by importance and limit
        edges_by_importance.sort(reverse=True)
        max_edges = min(100, len(edges_by_importance))  # Limit to 100 edges
        
        for _, i, j, from_name, to_name in edges_by_importance[:max_edges]:
            lines.append(f"    M{i} --> M{j}")
            edges_added += 1
        
        # Add styling for different layers
        lines.append("")
        lines.append("    %% Styling")
        for layer, color in [("Frontend", "#e1f5ff"), ("Backend", "#fff3e0"), 
                              ("Database", "#f3e5f5"), ("API", "#e8f5e9"),
                              ("Common", "#fce4ec")]:
            layer_modules = grouped_modules.get(layer, [])
            for module in layer_modules:
                if module['name'] in module_index:
                    idx = module_index[module['name']]
                    lines.append(f"    style M{idx} fill:{color}")
        
        # Add note about filtering if we limited nodes
        if len(modules) > len(important_modules):
            lines.append(f"    %% Showing {len(important_modules)} of {len(modules)} modules")
            lines.append(f"    %% ({edges_added} relationships)")
        elif edges_added == 0:
            lines.append("    %% Note: No internal dependencies detected")
            if unmatched_deps:
                # Show sample of unmatched deps for debugging
                sample_deps = list(unmatched_deps)[:5]
                lines.append(f"    %% Unmatched dependencies (sample): {', '.join(sample_deps)}")
        
        lines.append("```")
        return "\n".join(lines)
    
    def _group_modules_by_layer(self, modules: List[Dict[str, Any]]) -> Dict[str, List[Dict[str, Any]]]:
        """Group modules by their layer/directory."""
        from pathlib import Path
        
        groups = {
            "Frontend": [],
            "Backend": [],
            "Database": [],
            "API": [],
            "Common": [],
            "Tests": [],
            "Other": []
        }
        
        for module in modules:
            path = module.get('path', '')
            path_lower = path.lower()
            
            # Categorize based on path
            if any(x in path_lower for x in ['frontend', 'client', 'ui', 'components', 'pages', 'src/components']):
                groups["Frontend"].append(module)
            elif any(x in path_lower for x in ['backend', 'server', 'controllers', 'routes', 'middleware']):
                groups["Backend"].append(module)
            elif any(x in path_lower for x in ['models', 'migrations', 'database', 'db', 'schema']):
                groups["Database"].append(module)
            elif any(x in path_lower for x in ['api', 'services', 'providers']):
                groups["API"].append(module)
            elif any(x in path_lower for x in ['common', 'shared', 'utils', 'helpers', 'lib']):
                groups["Common"].append(module)
            elif any(x in path_lower for x in ['test', 'spec', '__tests__']):
                groups["Tests"].append(module)
            else:
                groups["Other"].append(module)
        
        # Remove empty groups
        return {k: v for k, v in groups.items() if v}
    
    def _select_important_modules(self, modules: List[Dict[str, Any]], max_nodes: int = 50) -> List[Dict[str, Any]]:
        """Select the most important modules to display."""
        # Score modules by:
        # 1. Number of dependencies (high = hub)
        # 2. Number of lines (complexity)
        # 3. Number of classes/functions
        
        scored_modules = []
        for module in modules:
            score = 0
            
            # Dependency count (both incoming and outgoing)
            deps = module.get('dependencies', [])
            score += len(deps) * 3  # Outgoing deps
            
            # Incoming dependencies (how many modules depend on this one)
            module_name = module.get('name', '')
            for other in modules:
                if module_name in other.get('dependencies', []):
                    score += 5  # Incoming deps are more important
            
            # Complexity indicators
            score += min(module.get('lines', 0) // 100, 10)  # Lines (capped)
            score += len(module.get('classes', [])) * 4
            score += len(module.get('functions', [])) * 2
            
            # Exclude test files from main diagram
            if 'test' not in module.get('path', '').lower():
                scored_modules.append((score, module))
        
        # Sort by score and take top N
        scored_modules.sort(reverse=True, key=lambda x: x[0])
        return [m for _, m in scored_modules[:max_nodes]]
    
    def _get_node_style(self, module: Dict[str, Any]) -> str:
        """Get visual style for a node based on its characteristics."""
        # Different shapes for different types
        classes = module.get('classes', [])
        functions = module.get('functions', [])
        
        if len(classes) > 3:
            return ":::classModule"
        elif len(functions) > 5:
            return ":::functionModule"
        else:
            return ""
    
    def generate_graph_diagram(
        self,
        modules: List[Dict[str, Any]],
        focus_module: Optional[str] = None,
        depth: int = 2
    ) -> str:
        """
        Generate a focused dependency graph around a specific module.
        
        Args:
            modules: List of module information
            focus_module: Name of module to focus on (shows its dependencies)
            depth: How many levels deep to traverse
            
        Returns:
            Mermaid diagram as string
        """
        if not modules:
            return "```mermaid\ngraph TD\n    Empty[No modules to display]\n```"
        
        # Build dependency graph
        module_map = {m['name']: m for m in modules}
        
        # If no focus module, pick the most connected one
        if not focus_module:
            focus_module = max(modules, 
                             key=lambda m: len(m.get('dependencies', [])))['name']
        
        # BFS to find connected modules within depth
        visited = {focus_module}
        to_visit = [(focus_module, 0)]
        selected_modules = []
        
        while to_visit:
            current, current_depth = to_visit.pop(0)
            if current not in module_map:
                continue
                
            selected_modules.append(module_map[current])
            
            if current_depth < depth:
                # Add dependencies
                deps = module_map[current].get('dependencies', [])
                for dep in deps:
                    if dep not in visited and dep in module_map:
                        visited.add(dep)
                        to_visit.append((dep, current_depth + 1))
        
        # Generate diagram
        lines = [
            "```mermaid",
            "graph TD",
            f"    %% Dependency graph focused on: {focus_module}",
            ""
        ]
        
        # Create index for selected modules
        module_index = {m['name']: i for i, m in enumerate(selected_modules)}
        
        # Add nodes
        for i, module in enumerate(selected_modules):
            node_id = f"M{i}"
            name = module['name']
            
            # Highlight the focus module
            if name == focus_module:
                lines.append(f"    {node_id}[{name}]:::focus")
            else:
                lines.append(f"    {node_id}[{name}]")
        
        lines.append("")
        
        # Add edges
        for i, module in enumerate(selected_modules):
            deps = module.get('dependencies', [])
            for dep in deps:
                if dep in module_index:
                    j = module_index[dep]
                    lines.append(f"    M{i} --> M{j}")
        
        # Add styling
        lines.append("")
        lines.append("    classDef focus fill:#ffeb3b,stroke:#f57c00,stroke-width:3px")
        
        lines.append("```")
        return "\n".join(lines)
    
    def generate_class_diagram(
        self,
        classes: List[Dict[str, Any]],
        title: str = "Class Diagram"
    ) -> str:
        """
        Generate class diagram.
        
        Args:
            classes: List of class information
            title: Diagram title
            
        Returns:
            Mermaid diagram as string
        """
        lines = [
            "```mermaid",
            "classDiagram",
            ""
        ]
        
        for cls in classes:
            class_name = cls.get('name', 'Unknown')
            lines.append(f"    class {class_name} {{")
            
            # Add attributes
            attributes = cls.get('attributes', [])
            for attr in attributes:
                lines.append(f"        {attr}")
            
            # Add methods
            methods = cls.get('methods', [])
            for method in methods:
                lines.append(f"        {method}()")
            
            lines.append("    }")
            lines.append("")
        
        # Add inheritance/relationships
        for cls in classes:
            class_name = cls.get('name', 'Unknown')
            
            # Inheritance
            parent = cls.get('parent')
            if parent:
                lines.append(f"    {parent} <|-- {class_name}")
            
            # Associations
            associations = cls.get('associations', [])
            for assoc in associations:
                lines.append(f"    {class_name} --> {assoc}")
        
        lines.append("```")
        return "\n".join(lines)
    
    def generate_flow_diagram(
        self,
        steps: List[Dict[str, Any]],
        title: str = "Process Flow"
    ) -> str:
        """
        Generate process flow diagram.
        
        Args:
            steps: List of process steps
            title: Diagram title
            
        Returns:
            Mermaid diagram as string
        """
        lines = [
            "```mermaid",
            "flowchart TD",
            f"    Start([Start])",
            ""
        ]
        
        # Add steps
        prev_id = "Start"
        for i, step in enumerate(steps):
            step_id = f"S{i}"
            step_text = step.get('text', f'Step {i+1}')
            step_type = step.get('type', 'process')
            
            # Different shapes for different step types
            if step_type == 'decision':
                lines.append(f"    {step_id}{{{step_text}}}")
            elif step_type == 'data':
                lines.append(f"    {step_id}[/{step_text}/]")
            else:
                lines.append(f"    {step_id}[{step_text}]")
            
            # Connect to previous
            lines.append(f"    {prev_id} --> {step_id}")
            prev_id = step_id
        
        # Add end
        lines.append(f"    {prev_id} --> End([End])")
        lines.append("```")
        return "\n".join(lines)
    
    def generate_sequence_diagram(
        self,
        interactions: List[Dict[str, Any]],
        title: str = "Sequence Diagram"
    ) -> str:
        """
        Generate sequence diagram.
        
        Args:
            interactions: List of interactions between components
            title: Diagram title
            
        Returns:
            Mermaid diagram as string
        """
        lines = [
            "```mermaid",
            "sequenceDiagram",
            ""
        ]
        
        # Extract participants
        participants = set()
        for interaction in interactions:
            participants.add(interaction.get('from', 'Unknown'))
            participants.add(interaction.get('to', 'Unknown'))
        
        # Declare participants
        for participant in sorted(participants):
            lines.append(f"    participant {participant}")
        
        lines.append("")
        
        # Add interactions
        for interaction in interactions:
            from_p = interaction.get('from', 'Unknown')
            to_p = interaction.get('to', 'Unknown')
            message = interaction.get('message', '')
            
            lines.append(f"    {from_p}->>+{to_p}: {message}")
            
            # Add response if available
            response = interaction.get('response')
            if response:
                lines.append(f"    {to_p}-->>-{from_p}: {response}")
        
        lines.append("```")
        return "\n".join(lines)
    
    def generate_entity_relationship_diagram(
        self,
        entities: List[Dict[str, Any]],
        relationships: List[Dict[str, Any]],
        title: str = "Entity Relationship Diagram"
    ) -> str:
        """
        Generate entity relationship diagram.
        
        Args:
            entities: List of entities
            relationships: List of relationships
            title: Diagram title
            
        Returns:
            Mermaid diagram as string
        """
        lines = [
            "```mermaid",
            "erDiagram",
            ""
        ]
        
        # Add entities with attributes
        for entity in entities:
            entity_name = entity.get('name', 'Unknown')
            attributes = entity.get('attributes', [])
            
            lines.append(f"    {entity_name} {{")
            for attr in attributes:
                attr_name = attr.get('name', 'field')
                attr_type = attr.get('type', 'string')
                lines.append(f"        {attr_type} {attr_name}")
            lines.append("    }")
            lines.append("")
        
        # Add relationships
        for rel in relationships:
            from_e = rel.get('from', 'Unknown')
            to_e = rel.get('to', 'Unknown')
            rel_type = rel.get('type', '||--||')  # one-to-one
            label = rel.get('label', '')
            
            lines.append(f"    {from_e} {rel_type} {to_e} : {label}")
        
        lines.append("```")
        return "\n".join(lines)



"""Factory for creating LLM providers."""

from typing import Optional
from sourcescribe.api.base import BaseLLMProvider
from sourcescribe.api.anthropic_provider import AnthropicProvider
from sourcescribe.api.openai_provider import OpenAIProvider
from sourcescribe.api.ollama_provider import OllamaProvider
from sourcescribe.config.models import LLMConfig, LLMProvider as LLMProviderEnum


class LLMProviderFactory:
    """Factory for creating LLM provider instances."""
    
    @staticmethod
    def create(config: LLMConfig) -> BaseLLMProvider:
        """
        Create an LLM provider based on configuration.
        
        Args:
            config: LLM configuration
            
        Returns:
            Instantiated provider
            
        Raises:
            ValueError: If provider is not supported
        """
        provider_map = {
            LLMProviderEnum.ANTHROPIC: AnthropicProvider,
            LLMProviderEnum.OPENAI: OpenAIProvider,
            LLMProviderEnum.OLLAMA: OllamaProvider,
        }
        
        provider_class = provider_map.get(config.provider)
        if not provider_class:
            raise ValueError(f"Unsupported LLM provider: {config.provider}")
        
        return provider_class(
            api_key=config.api_key,
            model=config.model,
            temperature=config.temperature,
            max_tokens=config.max_tokens,
            timeout=config.timeout,
            base_url=config.base_url,
        )
    
    @staticmethod
    def create_from_name(
        provider_name: str,
        api_key: Optional[str] = None,
        model: Optional[str] = None,
        **kwargs
    ) -> BaseLLMProvider:
        """
        Create an LLM provider by name.
        
        Args:
            provider_name: Name of the provider (anthropic, openai, ollama)
            api_key: Optional API key
            model: Optional model name
            **kwargs: Additional parameters
            
        Returns:
            Instantiated provider
        """
        provider_name = provider_name.lower()
        
        if provider_name == "anthropic":
            return AnthropicProvider(api_key=api_key, model=model or "", **kwargs)
        elif provider_name == "openai":
            return OpenAIProvider(api_key=api_key, model=model or "", **kwargs)
        elif provider_name == "ollama":
            return OllamaProvider(model=model or "llama2", **kwargs)
        else:
            raise ValueError(f"Unsupported provider: {provider_name}")



"""Tests for diagram generation."""

import pytest
from sourcescribe.engine.diagram import DiagramGenerator


def test_architecture_diagram():
    """Test generating architecture diagram."""
    generator = DiagramGenerator()
    
    modules = [
        {
            'name': 'ModuleA', 
            'dependencies': ['ModuleB'],
            'path': '/project/src/ModuleA.js',
            'lines': 100,
            'classes': [],
            'functions': []
        },
        {
            'name': 'ModuleB', 
            'dependencies': [],
            'path': '/project/src/ModuleB.js',
            'lines': 50,
            'classes': [],
            'functions': []
        },
    ]
    
    diagram = generator.generate_architecture_diagram(modules, "Test Architecture")
    
    assert '```mermaid' in diagram
    assert 'graph TD' in diagram
    assert 'ModuleA' in diagram
    assert 'ModuleB' in diagram
    assert 'M0 --> M1' in diagram or 'M1 --> M0' in diagram  # Should have a relationship


def test_class_diagram():
    """Test generating class diagram."""
    generator = DiagramGenerator()
    
    classes = [
        {
            'name': 'MyClass',
            'attributes': ['x', 'y'],
            'methods': ['doSomething', 'doAnother'],
        }
    ]
    
    diagram = generator.generate_class_diagram(classes)
    
    assert '```mermaid' in diagram
    assert 'classDiagram' in diagram
    assert 'MyClass' in diagram
    assert 'doSomething' in diagram


def test_flow_diagram():
    """Test generating flow diagram."""
    generator = DiagramGenerator()
    
    steps = [
        {'text': 'Initialize', 'type': 'process'},
        {'text': 'Check condition', 'type': 'decision'},
        {'text': 'Process data', 'type': 'process'},
    ]
    
    diagram = generator.generate_flow_diagram(steps)
    
    assert '```mermaid' in diagram
    assert 'flowchart TD' in diagram
    assert 'Initialize' in diagram


def test_sequence_diagram():
    """Test generating sequence diagram."""
    generator = DiagramGenerator()
    
    interactions = [
        {'from': 'Client', 'to': 'Server', 'message': 'Request'},
        {'from': 'Server', 'to': 'Database', 'message': 'Query'},
    ]
    
    diagram = generator.generate_sequence_diagram(interactions)
    
    assert '```mermaid' in diagram
    assert 'sequenceDiagram' in diagram
    assert 'Client' in diagram
    assert 'Server' in diagram


def test_module_grouping():
    """Test that modules are grouped by layer correctly."""
    generator = DiagramGenerator()
    
    modules = [
        {'name': 'UserComponent', 'path': '/frontend/components/User.js', 'dependencies': [], 'lines': 100, 'classes': [], 'functions': []},
        {'name': 'UserController', 'path': '/backend/controllers/user.js', 'dependencies': [], 'lines': 100, 'classes': [], 'functions': []},
        {'name': 'UserModel', 'path': '/models/User.js', 'dependencies': [], 'lines': 100, 'classes': [], 'functions': []},
        {'name': 'apiHelper', 'path': '/api/helper.js', 'dependencies': [], 'lines': 100, 'classes': [], 'functions': []},
    ]
    
    grouped = generator._group_modules_by_layer(modules)
    
    assert 'Frontend' in grouped
    assert 'Backend' in grouped
    assert 'Database' in grouped
    assert 'API' in grouped
    assert len(grouped['Frontend']) == 1
    assert len(grouped['Backend']) == 1


def test_important_module_selection():
    """Test that most important modules are selected."""
    generator = DiagramGenerator()
    
    # Create 100 modules with varying importance
    modules = []
    for i in range(100):
        modules.append({
            'name': f'Module{i}',
            'path': f'/src/Module{i}.js',
            'dependencies': [f'Module{j}' for j in range(i % 5)],  # Varying deps
            'lines': (i + 1) * 10,
            'classes': [],
            'functions': []
        })
    
    important = generator._select_important_modules(modules, max_nodes=10)
    
    # Should select only 10
    assert len(important) == 10
    
    # Should include modules with more dependencies and lines
    assert any(len(m.get('dependencies', [])) > 0 for m in important)


def test_graph_diagram():
    """Test focused dependency graph generation."""
    generator = DiagramGenerator()
    
    modules = [
        {'name': 'App', 'dependencies': ['Router', 'Config'], 'path': '/src/App.js', 'lines': 100, 'classes': [], 'functions': []},
        {'name': 'Router', 'dependencies': ['Pages'], 'path': '/src/Router.js', 'lines': 50, 'classes': [], 'functions': []},
        {'name': 'Pages', 'dependencies': [], 'path': '/src/Pages.js', 'lines': 200, 'classes': [], 'functions': []},
        {'name': 'Config', 'dependencies': [], 'path': '/src/Config.js', 'lines': 30, 'classes': [], 'functions': []},
    ]
    
    # Generate focused graph
    diagram = generator.generate_graph_diagram(modules, focus_module='App', depth=2)
    
    assert '```mermaid' in diagram
    assert 'graph TD' in diagram
    assert 'App' in diagram
    assert 'Router' in diagram
    assert 'focus' in diagram  # Focus module should have special styling


def test_large_module_filtering():
    """Test that large numbers of modules are filtered appropriately."""
    generator = DiagramGenerator()
    
    # Create 500 modules (like the user's case)
    modules = []
    for i in range(500):
        modules.append({
            'name': f'M{i}',
            'path': f'/src/M{i}.js',
            'dependencies': [f'M{(i+1) % 500}'] if i % 3 == 0 else [],
            'lines': 50,
            'classes': [],
            'functions': []
        })
    
    diagram = generator.generate_architecture_diagram(modules)
    
    assert '```mermaid' in diagram
    # Should mention filtering
    assert 'Showing' in diagram or 'modules' in diagram
    # Should not have 500 nodes
    node_count = diagram.count('[M')
    assert node_count <= 50  # Should limit to max_nodes



"""Setup script for SourceScribe (for backward compatibility)."""

from setuptools import setup, find_packages

# Read README
with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

# Read requirements
with open("requirements.txt", "r", encoding="utf-8") as f:
    requirements = [line.strip() for line in f if line.strip() and not line.startswith("#")]

setup(
    name="sourcescribe",
    version="0.1.0",
    author="SourceScribe Contributors",
    description="Auto-documentation engine using LLMs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/sourcescribe",
    packages=find_packages(exclude=["tests", "tests.*"]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    python_requires=">=3.7",
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "sourcescribe=sourcescribe.cli:main",
        ],
    },
)



"""OpenAI GPT API provider."""

from typing import List, Optional, Any, Iterator
from sourcescribe.api.base import BaseLLMProvider, LLMMessage, LLMResponse

try:
    import openai
    OPENAI_AVAILABLE = True
except ImportError:
    OPENAI_AVAILABLE = False


class OpenAIProvider(BaseLLMProvider):
    """OpenAI GPT provider."""
    
    def __init__(self, *args, **kwargs):
        """Initialize OpenAI provider."""
        if not OPENAI_AVAILABLE:
            raise ImportError(
                "OpenAI SDK not installed. Install with: pip install openai"
            )
        super().__init__(*args, **kwargs)
        
        if not self.api_key:
            raise ValueError("OPENAI_API_KEY is required")
        
        self.client = openai.OpenAI(
            api_key=self.api_key,
            timeout=self.timeout,
            base_url=self.base_url,
        )
        
        # Set default model if not provided
        if not self.model:
            self.model = "gpt-4-turbo-preview"
    
    def generate(
        self,
        messages: List[LLMMessage],
        system_prompt: Optional[str] = None,
        **kwargs: Any
    ) -> LLMResponse:
        """
        Generate response using GPT.
        
        Args:
            messages: Conversation messages
            system_prompt: System prompt
            **kwargs: Additional parameters
            
        Returns:
            LLMResponse
        """
        self.validate_config()
        
        # Convert messages to OpenAI format
        openai_messages = []
        
        if system_prompt:
            openai_messages.append({"role": "system", "content": system_prompt})
        
        for msg in messages:
            openai_messages.append({"role": msg.role, "content": msg.content})
        
        # Make API call
        response = self.client.chat.completions.create(
            model=self.model,
            messages=openai_messages,
            temperature=kwargs.get("temperature", self.temperature),
            max_tokens=kwargs.get("max_tokens", self.max_tokens),
        )
        
        choice = response.choices[0]
        
        return LLMResponse(
            content=choice.message.content or "",
            model=response.model,
            usage={
                "prompt_tokens": response.usage.prompt_tokens,
                "completion_tokens": response.usage.completion_tokens,
                "total_tokens": response.usage.total_tokens,
            },
            finish_reason=choice.finish_reason,
            raw_response=response,
        )
    
    def generate_streaming(
        self,
        messages: List[LLMMessage],
        system_prompt: Optional[str] = None,
        **kwargs: Any
    ) -> Iterator[str]:
        """
        Generate streaming response using GPT.
        
        Args:
            messages: Conversation messages
            system_prompt: System prompt
            **kwargs: Additional parameters
            
        Yields:
            Response chunks
        """
        self.validate_config()
        
        # Convert messages to OpenAI format
        openai_messages = []
        
        if system_prompt:
            openai_messages.append({"role": "system", "content": system_prompt})
        
        for msg in messages:
            openai_messages.append({"role": msg.role, "content": msg.content})
        
        # Stream response
        stream = self.client.chat.completions.create(
            model=self.model,
            messages=openai_messages,
            temperature=kwargs.get("temperature", self.temperature),
            max_tokens=kwargs.get("max_tokens", self.max_tokens),
            stream=True,
        )
        
        for chunk in stream:
            if chunk.choices[0].delta.content:
                yield chunk.choices[0].delta.content
